;; Copyright 2015, 2016, 2019 Kevin Ryde

;; This file is part of gp-inline.
;;
;; gp-inline is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; gp-inline is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with gp-inline; see the file COPYING.  If not, go to
;; <http://www.gnu.org/licenses/>.


(let ((dir (expand-file-name "..")))
  (message "load directory %S" dir)
  (setq load-path (cons dir load-path)))

;; Something in tex-mode of XEmacs 21.4.5 doesn't like to run under -batch,
;; giving "Wrong type argument: number-char-or-marker-p, nil".  Use this
;; dummy to have the desired comment syntax for the test.
;;
(defun my-dummy-tex-mode ()
  (interactive)
  (kill-all-local-variables)
  (set (make-local-variable 'comment-start-skip) "%+[ \t]*")
  (set (make-local-variable 'comment-start)      "%")
  (set (make-local-variable 'comment-end)        "")
  (set (make-local-variable 'comment-style)      'plain)
  (setq major-mode 'my-dummy-tex-mode
        mode-name  "my-dummy-tex-mode"))

(defun my-run-test ()

  (let ((count 0)
        (tempbuf "*t-gp-inline--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    ;; emacs24 `generate-file-autoloads' wants to be in a file-visiting
    ;; buffer
    (require 'autoload)
    (find-file "t-gp-inline--generated-loaddefs.el")
    (erase-buffer)
    (generate-file-autoloads "../gp-inline.el")
    (message "Evaluate autoloads")
    (eval-buffer)
    (save-buffer)
    (kill-buffer (current-buffer))
    (delete-file "t-gp-inline--generated-loaddefs.el")

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(gp-inline-defines
              gp-inline-eval))


    ;;-------------------------------------------------------------------------
    ;; byte compile state

    (require 'gp-inline)
    (message "`gp-inline-eval' is byte-code %S"
             (byte-code-function-p (symbol-function 'gp-inline-eval)))


    ;;-------------------------------------------------------------------------
    ;; gp-inline-prefix

    (emacs-lisp-mode) ;; for ";;" as comment-start-skip
    (mapcar (lambda (elem)
              (let* ((line (nth 0 elem))
                     (want (nth 1 elem))
                     (mode (nth 2 elem)))
                (erase-buffer)
                (insert line)
                (goto-char (point-min))
                (if mode (funcall mode))
                (let ((got-regexp (gp-inline-prefix-regexp)))
                  (unless (string-match (concat "^" got-regexp) want)
                    (error "gp-inline-prefix-regexp\nline %s\ngot  %S\nwant %S"
                           line got-regexp want))))
              (setq count (1+ count)))
            '(
              ("%o x=1" "%o")
              ("%o" "%o")
              ("%o (PARI)" "%o (PARI)")
              ("%o A123456 (PARI) a(n)=2*n" "%o A123456 (PARI)")

              ("123" "")
              ("GP-Test  123" "GP-Test")
              ("GP-Test-Last  123" "GP-Test-Last")
              ("GP-DEFINE  123" "GP-DEFINE")
              (";; GP-Define  123" ";; GP-Define")

              ("=for"    "=for")
              (" \t=for" "=for")
              ("=for GP-Test  2+2==4\n" "=for GP-Test")
              ("=for GP-Test" "=for GP-Test")

              ("% apply\n" "%"   my-dummy-tex-mode)
              (" #\t 123 */" "#")
              (" %   123 " "%")
              (" /* 123 */" "/*")
              ("  * GP-foo  123 " "* GP-foo")  ;; * only with GP-Something
              (" // 123 " "//")
              ;; (" \\\\ 123 " "\\\\")  ;; only in gp-script-mode
              ))

    ;;-------------------------------------------------------------------------
    ;; unload

    ;; couple of cycles
    (require 'gp-inline)
    (unload-feature 'gp-inline t)
    (require 'gp-inline)
    (unload-feature 'gp-inline t)
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))
